/*
 * Copyright 2023 Comcast Cable Communications Management, LLC
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * SPDX-License-Identifier: Apache-2.0
 */
import { isInteger } from './core/utils.js';
/**
 * Converts a color string to a color number value.
 */
export function hexColor(color = '') {
    if (isInteger(color)) {
        return color;
    }
    if (typeof color === 'string') {
        // Renderer expects RGBA values
        if (color.startsWith('#')) {
            return Number(color.replace('#', '0x') + (color.length === 7 ? 'ff' : ''));
        }
        if (color.startsWith('0x')) {
            return Number(color);
        }
        return Number('0x' + (color.length === 6 ? color + 'ff' : color));
    }
    return 0x00000000;
}
/**
 * Converts degrees to radians
 */
export function deg2rad(deg) {
    return (deg * Math.PI) / 180;
}
//# sourceMappingURL=utils.js.map