import { type Accessor } from 'solid-js';
import { ElementNode } from '@lightningjs/solid';
export interface KeyMap {
    ArrowLeft: 'Left';
    ArrowRight: 'Right';
    ArrowUp: 'Up';
    ArrowDown: 'Down';
    Enter: 'Enter';
    l: 'Last';
    ' ': 'Space';
    Backspace: 'Back';
    Escape: 'Escape';
    37: 'Left';
    39: 'Right';
    38: 'Up';
    40: 'Down';
    13: 'Enter';
    32: 'Space';
    8: 'Back';
    27: 'Escape';
}
export type KeyHandlerReturn = boolean | void;
export type KeyHandler = (this: ElementNode, e: KeyboardEvent, target: ElementNode) => KeyHandlerReturn;
/**
 * Generates a map of event handlers for each key in the KeyMap
 */
type valueof<T> = T[keyof T];
type KeyMapEventHandlers = {
    [K in valueof<KeyMap> as `on${Capitalize<K>}`]?: KeyHandler;
};
declare module '@lightningjs/solid' {
    /**
     * Augment the existing IntrinsicCommonProps interface with our own
     * FocusManager-specific properties.
     */
    interface IntrinsicCommonProps extends KeyMapEventHandlers {
        onFocus?: (currentFocusedElm: ElementNode | undefined, prevFocusedElm: ElementNode | undefined) => void;
        onBlur?: (currentFocusedElm: ElementNode | undefined, prevFocusedElm: ElementNode | undefined) => void;
        onKeyPress?: (this: ElementNode, e: KeyboardEvent, mappedKeyEvent: string | undefined, currentFocusedElm: ElementNode) => KeyHandlerReturn;
        onSelectedChanged?: (container: ElementNode, activeElm: ElementNode, selectedIndex: number | undefined, lastSelectedIndex: number | undefined) => void;
        skipFocus?: boolean;
        wrap?: boolean;
        plinko?: boolean;
    }
    interface IntrinsicNodeStyleProps {
        focus?: IntrinsicNodeStyleProps;
    }
    interface IntrinsicTextNodeStyleProps {
        focus?: IntrinsicTextNodeStyleProps;
    }
    interface TextNode {
        skipFocus?: undefined;
    }
}
declare const focusPath: Accessor<ElementNode[]>;
export { focusPath };
export declare const useFocusManager: (userKeyMap?: Partial<KeyMap>) => Accessor<ElementNode[]>;
