import { type Accessor } from 'solid-js';
import { ElementNode } from '@lightningjs/solid';
export interface DefaultKeyMap {
    Left: string | string[];
    Right: string | string[];
    Up: string | string[];
    Down: string | string[];
    Enter: string | string[];
    Last: string | string[];
}
export interface KeyMap extends DefaultKeyMap {
}
export type KeyHandlerReturn = boolean | void;
export type KeyHandler = (this: ElementNode, e: KeyboardEvent, target: ElementNode, handlerElm: ElementNode) => KeyHandlerReturn;
/**
 * Generates a map of event handlers for each key in the KeyMap
 */
type KeyMapEventHandlers = {
    [K in keyof KeyMap as `on${Capitalize<K>}`]?: KeyHandler;
};
declare module '@lightningjs/solid' {
    /**
     * Augment the existing IntrinsicCommonProps interface with our own
     * FocusManager-specific properties.
     */
    interface IntrinsicCommonProps extends KeyMapEventHandlers {
        onFocus?: (currentFocusedElm: ElementNode | undefined, prevFocusedElm: ElementNode | undefined) => void;
        onBlur?: (currentFocusedElm: ElementNode | undefined, prevFocusedElm: ElementNode | undefined) => void;
        onKeyPress?: (this: ElementNode, e: KeyboardEvent, mappedKeyEvent: string | undefined, handlerElm: ElementNode, currentFocusedElm: ElementNode) => KeyHandlerReturn;
        onSelectedChanged?: (container: ElementNode, activeElm: ElementNode, selectedIndex: number | undefined, lastSelectedIndex: number | undefined) => void;
        skipFocus?: boolean;
        wrap?: boolean;
        plinko?: boolean;
    }
    interface IntrinsicNodeStyleProps {
        focus?: IntrinsicNodeStyleProps;
    }
    interface IntrinsicTextNodeStyleProps {
        focus?: IntrinsicTextNodeStyleProps;
    }
    interface TextNode {
        skipFocus?: undefined;
    }
}
declare const focusPath: Accessor<ElementNode[]>;
export { focusPath };
export declare const useFocusManager: (userKeyMap?: Partial<KeyMap>) => Accessor<ElementNode[]>;
